#
import numpy
import pandas
from sklearn.metrics import roc_auc_score
#

from cr_tools.tests.utils import bayes_correction, concassive_shiels, variance_inflation_measure
#


def hh_index(sample):
    bins_counted = numpy.bincount(sample)
    indexed = (bins_counted / bins_counted.sum()) ** 2
    return indexed


def hl_statistic_bs(y_true_samples, y_hat_score_samples, ct):

    ns = numpy.array([y_true_samples[j].shape[0] for j in range(len(y_true_samples))])
    default_rate = sum([(y_true_samples[j] == 1).sum() for j in range(len(y_true_samples))]) / sum(ns)
    y_true_means = numpy.array([y_true_samples[j].mean() for j in range(len(y_true_samples))])
    y_hat_means = numpy.array([y_hat_score_samples[j].mean() for j in range(len(y_hat_score_samples))])
    y_true_means_bc = bayes_correction(array=y_true_means, ct=ct, dr=default_rate)
    stated = ((ns * ((y_true_means_bc - y_hat_means) ** 2)) / (y_hat_means * (1 - y_hat_means))).sum()
    return stated


def _hl_statistic(y_true_1c_numbers, y_true_0c_numbers, y_hat_1c_numbers, y_hat_0c_numbers):
    hl_stated = [(y_true_1c_numbers[j] - y_hat_1c_numbers[j]) ** 2 / y_hat_1c_numbers[j] +
                 (y_true_0c_numbers[j] - y_hat_0c_numbers[j]) ** 2 / y_hat_0c_numbers[j]
                 for j in range(len(y_true_1c_numbers))]

    hl_stated = sum(hl_stated)
    return hl_stated


def _hl_prepare(y_true_samples, y_hat_score_samples):
    y_true_1c_numbers = [(y_true_samples[j] == 1).sum() for j in range(len(y_true_samples))]
    y_true_0c_numbers = [(y_true_samples[j] == 0).sum() for j in range(len(y_true_samples))]
    y_hat_1c_numbers = [(y_hat_score_samples[j] == 1).sum() for j in range(len(y_true_samples))]
    y_hat_0c_numbers = [(y_hat_score_samples[j] == 0).sum() for j in range(len(y_true_samples))]

    return y_true_1c_numbers, y_true_0c_numbers, y_hat_1c_numbers, y_hat_0c_numbers


def hl_statistic(y_true_samples, y_hat_score_samples):

    y_true_1c_numbers, y_true_0c_numbers, y_hat_1c_numbers, y_hat_0c_numbers = _hl_prepare(y_true_samples,
                                                                                           y_hat_score_samples)
    stated = _hl_statistic(y_true_1c_numbers=y_true_1c_numbers, y_true_0c_numbers=y_true_0c_numbers,
                           y_hat_1c_numbers=y_hat_1c_numbers, y_hat_0c_numbers=y_hat_0c_numbers)

    return stated


def hl_statistic_with_binning(y_true_samples, y_hat_score_samples, n_bins):
    raise Exception("Not realized")


"""
def hl_statistic_with_binning(y_true_samples, y_hat_score_samples, n_bins):

    y_true_samples = binning(sample=y_true_samples, n_bins=n_bins)
    y_hat_score_samples = binning(sample=y_hat_score_samples, n_bins=n_bins)

    y_true_1c_numbers, y_true_0c_numbers, y_hat_1c_numbers, y_hat_0c_numbers = _hl_prepare(y_true_samples,
                                                                                           y_hat_score_samples)
    stated = _hl_statistic(y_true_1c_numbers=y_true_1c_numbers, y_true_0c_numbers=y_true_0c_numbers,
                           y_hat_1c_numbers=y_hat_1c_numbers, y_hat_0c_numbers=y_hat_0c_numbers)

    return stated
"""


def information_value(label_array, values_array, benchmark_label=0, bins=None, binning_array=None, n_bins=10):
    result = concassive_shiels(label_array, values_array, benchmark_label=benchmark_label, bins=bins,
                               binning_array=binning_array, n_bins=n_bins, woe=False)
    return result


def population_stability_index(actual, expected, bins=None, binning_on='expected', n_bins=10):
    values_array = numpy.concatenate((actual, expected))
    label_array = numpy.array([0] * actual.shape[0] + [1] * expected.shape[0])
    if bins is None:
        if binning_on == 'expected':
            binning_array = expected.copy()
        elif binning_on == 'actual':
            binning_array = actual.copy()
        elif binning_on == 'all':
            binning_array = numpy.concatenate((actual, expected))
        else:
            raise Exception("Misunderstood key value for 'binning_on' = {0}".format(binning_on))
    else:
        binning_array = None
    result = concassive_shiels(label_array, values_array, benchmark_label=0, bins=bins,
                               binning_array=binning_array, n_bins=n_bins, woe=False)
    return result


def variance_inflation_factor(model, model_kwargs, x, x_names=None):
    vif_s = []
    listed = list(range(x.shape[1]))
    for j in listed:
        x_, y_ = x[:, [k for k in listed if k != j]], x[:, j]
        model_ = model(**model_kwargs)
        model_.fit(X=x_, y=y_)
        y_hat = model_.predict(X=x_)
        vif_scored = variance_inflation_measure(y_true=y_, y_hat=y_hat)
        vif_s.append(vif_scored)
    if x_names is None:
        return vif_s
    else:
        report = pandas.DataFrame(data={'factor': x_names, 'VIF': vif_s})
        return report


def gini_coefficient(y_true, y_score):
    roc_auc_scored = roc_auc_score(y_true, y_score)
    gini_cff = 2 * roc_auc_scored - 1
    return gini_cff


def binomial_statistic(x):
    return x
