#
"""
Classes for testing
[Interfaced as: Test-Classes]
"""
#
from scipy.stats import chi2, ks_2samp, binom


#
from cr_tools.tests.utils import TestSkeleton
from cr_tools.tests.statistics import hl_statistic, hl_statistic_bs, hl_statistic_with_binning, gini_coefficient, binomial_statistic
from cr_tools.tests.confidence_intervals import binom_confidence


#
class HLTestBs(TestSkeleton):

    def __init__(self, n_bins, ct):

        n_degrees_of_freedom = n_bins - 2
        chi_squared = chi2(df=n_degrees_of_freedom).cdf

        super().__init__(statistic=hl_statistic_bs, cdf=chi_squared)
        self.ct = ct

    def calculate_statistic(self, y_true_binned, y_hat_score_binned):

        kwargs = {'y_true_samples': y_true_binned,
                  'y_hat_score_samples': y_hat_score_binned,
                  'ct': self.ct}
        stated = self._calculate_statistic(**kwargs)
        return stated

    def calculate_p_value(self, stated):

        p_value = 1 - self.cdf(stated)
        return p_value


class HLTestBinned(TestSkeleton):

    def __init__(self, n_bins):

        n_degrees_of_freedom = n_bins - 2
        chi_squared = chi2(df=n_degrees_of_freedom).cdf

        super().__init__(statistic=hl_statistic, cdf=chi_squared)

    def calculate_statistic(self, y_true_binned, y_hat_score_binned):

        kwargs = {'y_true_samples': y_true_binned,
                  'y_hat_score_samples': y_hat_score_binned}
        stated = self._calculate_statistic(**kwargs)
        return stated

    def calculate_p_value(self, stated):

        p_value = 1 - self.cdf(stated)
        return p_value


class HLTest(TestSkeleton):

    def __init__(self, n_bins=10):

        n_degrees_of_freedom = n_bins - 2
        chi_squared = chi2(df=n_degrees_of_freedom).cdf

        super().__init__(statistic=hl_statistic_with_binning, cdf=chi_squared)
        self.n_bins = n_bins

    def calculate_statistic(self, y_true_binned, y_hat_score_binned):

        kwargs = {'y_true_samples': y_true_binned,
                  'y_hat_score_samples': y_hat_score_binned,
                  'n_bins': self.n_bins}
        stated = self._calculate_statistic(**kwargs)
        return stated

    def calculate_p_value(self, stated):

        p_value = 1 - self.cdf(stated)
        return p_value


class KSTest:

    def __init__(self, alternative='two-sided'):

        self.alternative = alternative

    def calculate_statistic(self, actual, expected):
        stated, _ = ks_2samp(data1=actual, data2=expected, alternative=self.alternative)
        return stated

    def calculate_p_value(self, actual, expected):
        _, p_valued = ks_2samp(data1=actual, data2=expected, alternative=self.alternative)
        return p_valued


class GiniTest(TestSkeleton):

    def __init__(self):

        super().__init__(statistic=gini_coefficient, cdf=None)

    def calculate_statistic(self, y_true, y_score):

        kwargs = {'y_true': y_true,
                  'y_score': y_score}
        stated = self._calculate_statistic(**kwargs)
        return stated

    def calculate_p_value(self, y_true, y_score):

        raise Exception("Not implemented")


class BinomialTest(TestSkeleton):

    def __init__(self, p):

        cdf = binom(p).cdf  # ?
        super().__init__(statistic=binomial_statistic, cdf=cdf)

    def calculate_statistic(self, x):

        kwargs = {'x': x}
        stated = self._calculate_statistic(**kwargs)
        return stated

    def calculate_p_value(self, stated):

        p_val = 1 - self.cdf(stated)
        return p_val


class BinomialTestCI:

    def __init__(self):

        self._calculate_confidence_interval = binom_confidence

    def calculate_confidence_interval(self, y_score, rho, alpha):

        return self._calculate_confidence_interval(y_score, rho, alpha)

    def check(self, y_true, confidence_interval):

        value = y_true.sum() / y_true.shape[0]

        if value < confidence_interval[0]:
            return 'Lower'

        elif confidence_interval[0] <= value < confidence_interval[1]:
            return 'In'

        else:
            return 'Upper'

    def check_bc(self, y_true, confidence_interval, ct, dr):

        value = y_true.sum() / y_true.shape[0]
        forward = value * ct / dr
        backward = (1 - value) * (1 - ct) / (1 - dr)
        value_bc = forward / (forward + backward)

        if value_bc < confidence_interval[0]:
            return 'Lower'

        elif confidence_interval[0] <= value_bc < confidence_interval[1]:
            return 'In'

        else:
            return 'Upper'
